use eyre::Result;
use serde::Deserialize;
use std::path::Path;
use std::{env, fs};
use tracing::Level;

pub const CONFIG_FILE: &str = "config.identity.toml";

#[derive(Deserialize)]
pub struct Config {
    pub database_url: String,
    pub port: u16,
    pub domain: String,
    pub auto_migrate: Option<bool>,
    pub log_level: Option<String>,
}

impl Config {
    pub fn load() -> Result<Self> {
        let cwd = env::current_dir()?;
        let config_file = Path::join(cwd.as_path(), Path::new(CONFIG_FILE));
        println!("config file: {}", config_file.display());

        let s = fs::read_to_string(config_file)?;
        let config = toml::from_str(s.as_str())?;

        Ok(config)
    }

    pub fn tracing_level(&self) -> Option<Level> {
        match self
            .log_level
            .as_deref()
            .unwrap_or("INFO")
        {
            "TRACE" => Some(Level::TRACE),
            "DEBUG" => Some(Level::DEBUG),
            "INFO" => Some(Level::INFO),
            "WARN" => Some(Level::WARN),
            "ERROR" => Some(Level::ERROR),
            _ => None
        }
    }
}

impl Default for Config {
    fn default() -> Self {
        Config {
            database_url: env::var("DATABASE_URL").unwrap_or("".into()),
            domain: "".into(),
            port: 3000,
            auto_migrate: None,
            log_level: None,
        }
    }
}
