use axum::{async_trait, extract::FromRequestParts, http::request::Parts, Extension};
use foxchat::{http::ApiError, fed::SERVER_HEADER, FoxError};

use crate::{app_state::AppState, model::chat_instance::ChatInstance};
use std::sync::Arc;

pub struct ProxyServerHeader(pub ChatInstance);

#[async_trait]
impl<S> FromRequestParts<S> for ProxyServerHeader
where
    S: Send + Sync,
{
    type Rejection = ApiError;

    async fn from_request_parts(parts: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        let state: Extension<Arc<AppState>> = Extension::from_request_parts(parts, state)
            .await
            .expect("AppState was not added as an extension");

        let domain = parts
            .headers
            .get(SERVER_HEADER)
            .ok_or(FoxError::InvalidHeader)?
            .to_str()?;

        let instance = ChatInstance::get(state.0.clone(), domain).await?;

        Ok(ProxyServerHeader(instance))
    }
}
