use std::sync::Arc;

use axum::{Extension, Json};
use eyre::Result;
use foxchat::http::ApiError;
use serde::{Deserialize, Serialize};
use ulid::Ulid;

use crate::{app_state::AppState, db::{create_token, account::hash_password}};

pub async fn create_user(
    Extension(state): Extension<Arc<AppState>>,
    Json(data): Json<CreateUserRequest>,
) -> Result<Json<CreateUserResponse>, ApiError> {
    let mut tx = state.pool.begin().await?;

    let password_hash = hash_password(data.password)?;

    let account = sqlx::query!(
        "insert into accounts (id, username, email, password) values ($1, $2, $3, $4) returning id, username",
        Ulid::new().to_string(), data.username, data.email, password_hash)
            .fetch_one(&mut *tx)
            .await?;
    let token = create_token(&mut *tx, &account.id).await?;

    tx.commit().await?;

    Ok(Json(CreateUserResponse {
        id: account.id,
        username: account.username,
        token,
    }))
}

#[derive(Deserialize)]
pub struct CreateUserRequest {
    pub username: String,
    pub email: String,
    pub password: String,
}

#[derive(Serialize)]
pub struct CreateUserResponse {
    pub id: String,
    pub username: String,
    pub token: String,
}
