mod create_user;

use std::sync::Arc;

use axum::{
    extract::Path,
    routing::{get, post},
    Extension, Json, Router,
};
use foxchat::{http::ApiError, model::User};

use crate::{app_state::AppState, fed::FoxRequestData};

pub fn router() -> Router {
    Router::new()
        .route("/_fox/ident/users", post(create_user::create_user))
        .route("/_fox/ident/users/:id", get(get_user))
}

pub async fn get_user(
    Extension(state): Extension<Arc<AppState>>,
    _data: FoxRequestData,
    Path(id): Path<String>,
) -> Result<Json<User>, ApiError> {
    let user = sqlx::query!(
        "select id, username, avatar from accounts where id = $1",
        id
    )
    .fetch_one(&state.pool)
    .await?;

    Ok(Json(User {
        id: user.id,
        username: user.username,
        instance: state.config.domain.clone(),
        avatar_url: None,
    }))
}
