use std::sync::Arc;

use crate::{app_state::AppState, db::check_token, model::account::Account};
use axum::{
    async_trait,
    extract::FromRequestParts,
    http::{header::AUTHORIZATION, request::Parts},
    Extension,
};
use foxchat::{http::ApiError, FoxError};

pub struct AuthUser(pub Account);

#[async_trait]
impl<S> FromRequestParts<S> for AuthUser
where
    S: Send + Sync,
{
    type Rejection = ApiError;

    async fn from_request_parts(parts: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        let state: Extension<Arc<AppState>> = Extension::from_request_parts(parts, state)
            .await
            .expect("AppState was not added as an extension");

        let token = parts
            .headers
            .get(AUTHORIZATION)
            .ok_or(FoxError::Unauthorized)?
            .to_str()?
            .to_string();

        if let Ok(user) = check_token(&state.pool, token).await {
            return Ok(AuthUser(user));
        }

        Err(FoxError::Unauthorized.into())
    }
}
