use std::sync::Arc;

use axum::{http::StatusCode, Extension, Json};
use foxchat::{http::ApiError, s2s::http::RestEvent, FoxError};
use tracing::info;

use crate::{app_state::AppState, fed::FoxRequestData};

pub async fn post_inbox(
    Extension(state): Extension<Arc<AppState>>,
    request: FoxRequestData,
    Json(evt): Json<RestEvent>,
) -> Result<(StatusCode, &'static str), ApiError> {
    match evt {
        RestEvent::GuildJoin { guild_id, user_id } => {
            info!(
                "received GUILD_JOIN event from {}/{} for user {}/guild {}",
                &request.instance.id, &request.instance.domain, &user_id, &guild_id,
            );

            let mut tx = state.pool.begin().await?;

            sqlx::query!(
                r#"INSERT INTO chat_instance_accounts
                (account_id, chat_instance_id) VALUES ($1, $2)
                ON CONFLICT (account_id, chat_instance_id) DO NOTHING"#,
                &user_id,
                &request.instance.id
            )
            .execute(&mut *tx)
            .await?;

            sqlx::query!(
                r#"INSERT INTO guilds_accounts (chat_instance_id, guild_id, account_id)
                VALUES ($1, $2, $3) ON CONFLICT (chat_instance_id, guild_id, account_id) DO NOTHING"#,
                &request.instance.id,
                &guild_id,
                &user_id
            )
            .execute(&mut *tx)
            .await?;

            tx.commit().await?;
        }
        RestEvent::GuildLeave { guild_id, user_id } => {
            sqlx::query!(
                "DELETE FROM guilds_accounts WHERE chat_instance_id = $1 AND guild_id = $2 AND account_id = $3",
                &request.instance.id,
                &guild_id,
                &user_id,
            ).execute(&state.pool).await?;
        }
        _ => {
            return Err(FoxError::InvalidServer.into());
        }
    }

    Ok((StatusCode::NO_CONTENT, ""))
}
