mod account;
mod auth;
mod inbox;
mod node;
mod proxy;
mod ws;

use std::sync::Arc;

use axum::{routing::{get, post}, Extension, Router};
use sqlx::{Pool, Postgres};
use tower_http::trace::TraceLayer;

use crate::{app_state::AppState, config::Config, model::instance::Instance};

pub fn new(pool: Pool<Postgres>, config: Config, instance: Instance) -> Router {
    let app_state = Arc::new(AppState {
        pool,
        config,
        public_key: instance.public_key,
        private_key: instance.private_key,
    });

    let app = Router::new()
        .merge(account::router())
        .nest("/_fox/proxy", proxy::router())
        .route("/_fox/ident/node", get(node::get_node))
        .route("/_fox/ident/node/:domain", get(node::get_chat_node))
        .route("/_fox/ident/ws", get(ws::handler))
        .route("/_fox/ident/inbox", post(inbox::post_inbox))
        .layer(TraceLayer::new_for_http())
        .layer(Extension(app_state));

    return app;
}
