use std::sync::Arc;

use axum::{Extension, Json, extract::Path};
use eyre::{Context, Result};
use foxchat::{http::ApiError, s2s::http::NodeResponse};
use rsa::pkcs1::EncodeRsaPublicKey;

use crate::{app_state::AppState, model::chat_instance::ChatInstance};

pub async fn get_node(
    Extension(state): Extension<Arc<AppState>>,
) -> Result<Json<NodeResponse>, ApiError> {
    let public_key: String = state
        .public_key
        .to_pkcs1_pem(rsa::pkcs8::LineEnding::LF)
        .wrap_err("serializing public key")?;

    Ok(Json(NodeResponse {
        software: NODE_SOFTWARE.into(),
        public_key,
    }))
}

const NODE_SOFTWARE: &str = "foxchat_ident";

pub async fn get_chat_node(
    Extension(state): Extension<Arc<AppState>>,
    Path(domain): Path<String>,
) -> Result<Json<ChatInstance>, ApiError> {
    let instance = ChatInstance::get(state, &domain).await?;

    Ok(Json(instance))
}
